"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fork_ts_checker_webpack_plugin_1 = __importDefault(require("fork-ts-checker-webpack-plugin"));
var tsconfig_1 = require("tsconfig");
;
exports.default = (function (options) {
    return {
        config: getReactStaticConfigExtender(),
        webpack: getWebpackConfigExtender(options),
    };
});
;
function getWebpackConfigExtender(options) {
    // The `|| undefined` is there to compensate for `resolve` possibly returning
    // `void`, which TypeScript complains about not being `undefined`...
    var tsconfigPath = tsconfig_1.resolveSync('./', (options || {}).tsconfig) || /* istanbul ignore next */ undefined;
    return function (previousConfig, args) {
        var previousModuleConfig = previousConfig.module || {};
        var previousResolveConfig = previousConfig.resolve || {};
        var previousResolveExtensions = previousResolveConfig.extensions || [];
        var typescriptLoader = getTypeScriptLoader(args.defaultLoaders.jsLoader);
        var plugins = previousConfig.plugins || [];
        if ((options || {}).typeCheck !== false) {
            plugins.push(getTypecheckPlugin(tsconfigPath));
        }
        var loaders = (Array.isArray(previousModuleConfig.rules)
            && previousModuleConfig.rules.length > 0
            && Array.isArray(previousModuleConfig.rules[0].oneOf))
            ? previousModuleConfig.rules[0].oneOf.slice() : [args.defaultLoaders.jsLoader, args.defaultLoaders.jsLoaderExt, args.defaultLoaders.cssLoader, args.defaultLoaders.fileLoader];
        if (loaders.indexOf(args.defaultLoaders.jsLoader) !== -1) {
            // If the default Javascript loader is still present, replace it with the TypeScript loader:
            loaders[loaders.indexOf(args.defaultLoaders.jsLoader)] = typescriptLoader;
        }
        else {
            // Otherwise just add the TypeScript loader to the list of loaders, before the others:
            loaders.unshift(typescriptLoader);
        }
        return __assign({}, previousConfig, { plugins: plugins, resolve: __assign({}, previousResolveConfig, { extensions: previousResolveExtensions.concat(['.ts', '.tsx']) }), module: __assign({}, previousModuleConfig, { rules: [{
                        oneOf: loaders,
                    }] }) });
    };
}
function getReactStaticConfigExtender() {
    return function (previousConfig) {
        return __assign({}, previousConfig, { extensions: previousConfig.extensions.concat(['.ts', '.tsx']) });
    };
}
function getTypeScriptLoader(jsLoader) {
    var jsLoaderBabelPresets = jsLoader.use[0].options.presets || [];
    var typescriptLoader = __assign({}, jsLoader, { test: /\.(js|jsx|ts|tsx)$/, use: [
            __assign({}, jsLoader.use[0], { options: __assign({}, jsLoader.use[0].options, { presets: jsLoaderBabelPresets.concat('@babel/preset-typescript') }) }),
        ] });
    return typescriptLoader;
}
function getTypecheckPlugin(tsconfigPath) {
    return new fork_ts_checker_webpack_plugin_1.default({
        async: false,
        checkSyntacticErrors: true,
        tsconfig: tsconfigPath,
    });
}
